@@ -1,8 +1,11 @@
+{-# LANGUAGE CPP #-}
 module Main where
 
 import Control.Exception
 import Control.Concurrent
+#if __GLASGOW_HASKELL__ < 705
 import Prelude hiding (catch)
+#endif
 import System.Mem
 
 -- illustrates the BlockOnDeadMVar exception
