@@ -1,8 +1,11 @@
+{-# LANGUAGE CPP #-}
 module Main where
 
 import Control.Exception
 import Control.Concurrent
+#if __GLASGOW_HASKELL__ < 705
 import Prelude hiding (catch)
+#endif
 
 -- the BlockOnDeadMVar exception doesn't cause any output by default
 
