@@ -4,23 +4,24 @@
 module DefsVect where
 
 import Data.Array.Parallel
-import Data.Array.Parallel.Prelude.Bool
-import Data.Array.Parallel.Prelude.Int  (Int)
 
 
-{-# VECTORISE class Eq #-}
-{-# VECTORISE SCALAR instance Eq Int #-}
+-- {-# VECTORISE SCALAR instance Eq Char #-}
+-- {-# VECTORISE SCALAR instance Eq Float #-}
+-- {-# VECTORISE SCALAR instance Ord Char #-}
+-- {-# VECTORISE SCALAR instance Ord Float #-}
 
--- {-# VECTORISE class Ord #-}
--- {-# VECTORISE SCALAR instance Ord Int #-}
-
--- {-# VECTORISE type Ordering #-}
 
 data MyBool = MyTrue | MyFalse
 
 class Eq a => Cmp a where
   cmp :: a -> a -> Bool
 
+-- FIXME:
+-- instance Cmp Int where
+--   cmp = (==)
+
+-- isFive :: (Eq a, Num a) => a -> Bool
 isFive :: Int -> Bool
 isFive x = x == 5
 
@@ -49,4 +50,4 @@ isEqs :: PArray Int -> Bool
 isEqs xs = isEqs' (fromPArrayP xs)
 
 isEqs' :: [:Int:] -> Bool
-isEqs' xs = andP (mapP isEq xs)
\ No newline at end of file
+isEqs' xs = andP (mapP isEq xs)
