@@ -12,4 +12,4 @@ dotp :: PArray Double -> PArray Double -> Double
 dotp v w = dotp' (fromPArrayP v) (fromPArrayP w)
 
 dotp' :: [:Double:] -> [:Double:] -> Double
-dotp' v w = D.sumP (zipWithP (*) v w)
+dotp' v w = D.sumP (zipWithP (D.*) v w)
