@@ -30,6 +30,6 @@ log2 x = ceiling log_x
           log_x = logBase 2 (fromIntegral (max 1 x))
 
 vals = [1, 2, 17, 259, 1000, 10000,
-        2^30 + 9000, 2^31 - 1, 2^31, 2^31 + 1,
-        2^32 - 1, 2^32, 2^32 + 1]
+        2^30 + 9000, 2^31 - 1, 2^31 + 1,
+        2^32 - 1, 2^32 + 1]
 
